/*****************************************************************************
 * $Id: stack.cc,v 1.1 2006/10/21 00:19:08 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Test of Stack template class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "gfx_stack.hh"

/*****************************************************************************
 * Test.
 *****************************************************************************/
class Object
{
public:
	Object( int n ) { val = n; }
	int val;
};

namespace gfx {
bool
StackTest( void )
{
// -------------------------------------------
 {
	Object s1(1);
	Object s2(2);
	Object s3(3);

	Stack<Object>* stack = new Stack<Object>;

	stack->Push( &s1 ); if ( stack->Cnt() != 1 ) { Fail(); }
	stack->Push( &s2 ); if ( stack->Cnt() != 2 ) { Fail(); }
	stack->Push( &s3 ); if ( stack->Cnt() != 3 ) { Fail(); }

	Object* p = NULL;
	if ( stack->IfEmpty() ) { Fail(); }
	p = stack->Pop(); if ( ! p || p->val != 3 ) { Fail(); }
	p = stack->Pop(); if ( ! p || p->val != 2 ) { Fail(); }
	p = stack->Pop(); if ( ! p || p->val != 1 ) { Fail(); }
	if ( ! stack->IfEmpty() ) { Fail(); }
	if ( stack->Cnt() != 0 ) { Fail(); }
 }
// -------------------------------------------
 {
	Object s1(1);
	Object s2(2);
	Object s3(3);

	Stack<Object>* stack = new Stack<Object>;

	stack->Push( &s1 );
	stack->Push( &s2 );
		stack->Pop();		// difference vs prior test
	stack->Push( &s3 );

	Object* p = NULL;
	if ( stack->IfEmpty() ) { Fail(); }
	p = stack->Pop(); if ( ! p || p->val != 3 ) { Fail(); }
	p = stack->Pop(); if ( ! p || p->val != 1 ) { Fail(); }
	if ( ! stack->IfEmpty() ) { Fail(); }
 }
	return true;
}
} // namespace gfx

/*****************************************************************************
 * main.
 *****************************************************************************/
int
main( int argc, char** argv )
{
	GFX_INIT( argv[0] );
	int res = 1;
	if ( StackTest() )
	{
		cerr << "Test PASSED. \n";
		res = 0;
	}
	else
	{
		cerr << "Test FAILED! \n";
		res = 1;
	}
	GFX_CLEANUP();
	exit( res );
}
