/*****************************************************************************
 * $Id: eng_lod.cc,v 1.6 2006/11/08 17:43:48 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Level-of-detail.
 * @remarks	There are two kinds of LODs: the global user-selected LOD (LodUser)
 *          and the LOD relative to a particular Object (LodObj).
 *//*
 * LEGAL:	COPYRIGHT (C) 2005 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#define ENG_LOD_CC 1
#include "eng.hh"
#include "eng_lod.hh"

//////////////////////////////////////////////////////////////////////////////
//////////////////////////  User Level-of-detail class  //////////////////////
//////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * Set LOD.
 *****************************************************************************/
void
LodUser::Set( eLodUser lod )
{
	mLod = lod;
	gEngineEvent.mLodUserChange.Broadcast();	// broadcast that LOD was changed
}
void
LodUser::Reset( void )
{
	Set( eLodUser_DEFAULT );
}
void
LodUser::Inc( void )
{
	eLodUser lod = (eLodUser) (int(Get()) + 1);
	if ( lod > eLodUser_MAX ) lod = eLodUser_MIN;
	Set( lod );
}

/*****************************************************************************
 * LOD/string conversions.
 *****************************************************************************/
static const string sLodUserStrings[] = { "MIN", "LOW", "MID", "HIGH", "MAX" };
const string&
LodUser::Lod2String( eLodUser lod )
{
	ASSERT( lod <= eLodUser_MAX );
	return sLodUserStrings[int(lod)];
}
eLodUser
LodUser::String2Lod( const string& lod )
{
	for ( uint i = 0; i < ELEMS(sLodUserStrings); ++i )
	{
		if ( lod == sLodUserStrings[i] ) return (eLodUser)i;
	}
	ASSERT_OR_RETURN_VAL( 0, eLodUser_MID );
}
